# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Define entrypoints.
"""

import argparse

from lasersafety import DISCLAIMER, __version__
from lasersafety.laser import LaserBeam


def _create_parsers() -> argparse.ArgumentParser:
    """Create the parsers.

    Returns:
        argparse.ArgumentParser: the main parser.
    """
    parser = argparse.ArgumentParser(prog="lasersafety")

    parser.add_argument("--version", action="version", version=__version__)

    parser.add_argument(
        "-p",
        "--avg-power",
        type=float,
        help="Average power of the laser",
    )

    parser.add_argument(
        "-s",
        "--peak-power",
        type=float,
        help="Peak power of the laser",
    )

    parser.add_argument(
        "-r",
        "--rep-rate",
        type=float,
        help="Repetition rate of the laser",
    )

    parser.add_argument(
        "-t",
        "--pulse-duration",
        type=float,
        help="Pulse duration of the laser",
    )

    parser.add_argument(
        "-e",
        "--pulse-energy",
        type=float,
        help="Pulse energy of the laser",
    )

    parser.add_argument(
        "-w",
        "--wavelength",
        type=float,
        help="Wavelength of the laser",
    )

    parser.add_argument(
        "-d",
        "--beam-diameter",
        type=float,
        help="Beam diameter of the laser",
    )

    parser.add_argument(
        "--divergence",
        type=float,
        help="Beam divergence of the laser",
    )

    subparsers = parser.add_subparsers()

    en207_parser = subparsers.add_parser(
        "en207", help="Perform the computation for the EN207 standard"
    )
    en207_parser.set_defaults(func=en207)

    en208_parser = subparsers.add_parser(
        "en208", help="Perform the computation for the EN208 standard"
    )
    en208_parser.set_defaults(func=en208)

    return parser


def en207(args: argparse.Namespace):
    """EN207 analyis entrypoint.

    Args:
        args (argparse.Namespace): arguments passed to the command line.
    """
    laser = LaserBeam(
        wavelength=args.wavelength,
        repetition_rate=args.rep_rate,
        beam_diameter=args.beam_diameter,
        average_power=args.avg_power,
        pulse_duration=args.pulse_duration,
        pulse_energy=args.pulse_energy,
        divergence=args.divergence,
    )

    print(40 * "-")

    print("Laser information")
    print(laser)

    print(40 * "-")

    print("EN207 analysis\n")

    for mode, level in laser.en207_analysis():
        print(f"{laser.wavelength*1e9} nm", mode.value, f"LB{str(level)}")


def en208(args: argparse.Namespace):
    """EN208 analyis entrypoint.

    Args:
        args (argparse.Namespace): arguments passed to the command line.
    """
    laser = LaserBeam(
        wavelength=args.wavelength,
        repetition_rate=args.rep_rate,
        beam_diameter=args.beam_diameter,
        average_power=args.avg_power,
        pulse_duration=args.pulse_duration,
        pulse_energy=args.pulse_energy,
        divergence=args.divergence,
    )

    print(40 * "-")

    print("Laser information")
    print(laser)

    print(40 * "-")

    print("EN208 analysis\n")

    print(f"{laser.wavelength*1e9} nm", f"RB{str(laser.en208_analysis())}")


def main():
    """
    Main entrypoint.
    """
    parser = _create_parsers()

    args = parser.parse_args()

    print(DISCLAIMER)

    if hasattr(args, "func"):
        args.func(args)
    else:
        print("No command specified. Run with -h|--help to see the possible commands.")


if __name__ == "__main__":
    main()



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Utility functions to validate user inputs.
"""

from typing import Union

import numpy as np


def validate_wavelength_en207(wavelength: Union[float, int]) -> None:
    """Check that the wavelength is compatible with the EN207 standard.

    Args:
        wavelength (Union[float, int]): the wavelength to validate, in m.

    Raises:
        ValueError: if the wavelength is not a float or an int.
        ValueError: if the wavelength is outside the range 180 nm - 1 mm.
    """
    # Check type
    if not isinstance(wavelength, (float, int)):
        raise ValueError("wavelength should be a float (or int)")

    # Check value per EN207 standard
    if wavelength < 180e-9 or wavelength > 1e-3:
        raise ValueError(
            f"EN207 standards is only valid for wavelengths between 180nm and 1mm (got {wavelength})"
        )


def validate_wavelength_en208(wavelength: Union[float, int]) -> None:
    """Check that the wavelength is compatible with the EN208 standard.

    Args:
        wavelength (Union[float, int]): the wavelength to validate, in m.

    Raises:
        ValueError: if the wavelength is not a float or an int.
        ValueError: if the wavelength is outside the range 400 nm - 700 nm.
    """
    # Check type
    if not isinstance(wavelength, (float, int)):
        raise ValueError("wavelength should be a float (or int)")

    # Check value per EN208 standard
    if wavelength < 400e-9 or wavelength > 700e-9:
        raise ValueError(
            f"EN208 standards is only valid for wavelengths between 400nm and 700mm (got {wavelength})"
        )


def validate_strictly_positive_number(
    number: Union[float, int], parameter_name="parameter"
) -> None:
    """Check that the input number is stritcly positive.

    Args:
        number (Union[float, int]): the number to validate.
        parameter_name (str, optional): the parameter, for raising exceptions. Defaults to "parameter".

    Raises:
        ValueError: if the number is not a float or an int.
        ValueError: if the number is lower or equal to 0.
    """
    # Check type
    if not isinstance(number, (float, int)):
        raise ValueError(f"{parameter_name} should be a float or int")

    # Check value is strictly positive
    if number <= 0:
        raise ValueError(f"{parameter_name} should be striclty positive (got {number})")


def format_value(value: float | int) -> str:
    """Format a scientific value with scientific prefixes.

    It is compatible with prefixes from femto to Giga.

    Args:
        value (float | int): the value to format.

    Raises:
        ValueError: if the input value is not a float or an int.

    Returns:
        str: the formatted scientific value.
    """
    if not isinstance(value, (float, int)):
        raise ValueError("format_value can only be applied to a float or an int")

    if value == 0:
        return "0"

    magnitude = np.log10(np.abs(value))

    magnitude_suffix = ["G", "M", "k", "", "m", "μ", "n", "p", "f"]

    for i, magnitude_threshold in enumerate([9, 6, 3, 0, -3, -6, -9, -12, -15]):
        if magnitude >= magnitude_threshold:
            return str(value * 10 ** (-magnitude_threshold)) + " " + magnitude_suffix[i]

    return str(value)



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
A library for laser safety.
"""

__version__ = "0.9.0"

DISCLAIMER = """This code is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

In particular, you should ALWAYS check the results provided by this code before using them for
actual laser safety.
"""



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Defines a laser beam object with the important parameters to make the laser safety computations.
"""

from typing import Optional, Tuple, List

import numpy as np

from lasersafety.modes import get_test_by_pulse_duration, Mode
from lasersafety.utils import format_value
from lasersafety.en207 import continuous_scale_number, pulsed_scale_number
from lasersafety.en208 import continuous_scale_number_en208, pulsed_scale_number_en208


# pylint: disable=too-many-instance-attributes
class LaserBeam:
    """
    The LaserBeam object with the necessary parameters to make the laser safety computations.
    """

    average_power: Optional[float | int]  #: Average power of the beam, in W.
    repetition_rate: Optional[float | int]  #: Repetition rate of the laser, in Hz.
    pulse_duration: Optional[float | int]  #: Pulse duration for pulsed lasers, in s.
    pulse_energy: Optional[float | int]  #: Pulse energy for pulsed lasers, in J.
    peak_power: Optional[float | int]  #: Peak power for pulsed lasers, in W.
    wavelength: Optional[float | int]  #: Wavelength of the beam, in m.
    base_beam_diameter: Optional[float | int]  #: Base beam diameter, in m.
    beam_divergence: Optional[float | int]  #: Beam divergence, in rad.
    distance: Optional[float | int]  #: Distance to consider for divergent beam, in m.
    continuous: bool  #: True, if the laser is continuous, False if it is pulsed.

    # pylint: disable=too-many-arguments, too-many-positional-arguments
    def __init__(
        self,
        wavelength: float | int,
        repetition_rate: float | int,
        beam_diameter: float | int,
        average_power: Optional[float | int] = None,
        peak_power: Optional[float | int] = None,
        pulse_duration: Optional[float | int] = None,
        pulse_energy: Optional[float | int] = None,
        divergence: Optional[float | int] = 0,
        distance: float | int = 10e-2,
    ):
        """
        The init method has four optional parameters: average_power, peak_power, pulse_duration, and pulse_energy.
        These four parameters are interconnected, and usually, two of them are enough to find the other two
        (unless the average power and pulse energy are given). Creating a laser beam with two few OR too many
        arguments (or only the average power and pulse energy), will result in a raised exception.

        Args:
            wavelength (float | int): wavelength of the light, in m.
            repetition_rate (float | int): repetition rate for pulsed lasers, in Hz. Set this value to 0 for a continuous laser.
            beam_diameter (float | int): beam diameter, in m.
            average_power (Optional[float  |  int], optional): average power, in W. This parameter may be omitted if the peak_power and pulse_duration or the pulse energy are given. This parameter may not be omitted for continuous lasers. Defaults to None.
            peak_power (Optional[float  |  int], optional): peak power, in W. This parameter may be omitted if the average_power and pulse_duration or the pulse_energy and pulse_duration are given. This parameter may be omitted for pulsed lasers. Defaults to None.
            pulse_duration (Optional[float  |  int], optional): pulse duration, in s. This parameter may be omitted if the average_power or the pulse_energy and pulse_duration are given. This parameter may be omitted for pulsed lasers. Defaults to None.
            pulse_energy (Optional[float  |  int], optional): pulse energy, in J. This parameter may be omitted if the peak_power and pulse_duration or average power are given. This parameter may be omitted for pulsed lasers. Defaults to None.
            divergence (float | int, optional): divergence, in rad. Defaults to 0.
            distance (float | int, optional): distance to consider for divergent beam, in m. Defaults to 10e-2.

        Raises:
            ValueError: if the average power is omitted for a continuous laser.
        """
        self.wavelength = wavelength
        self.repetition_rate = repetition_rate

        self.base_beam_diameter = beam_diameter
        if divergence is None:
            self.beam_divergence = 0
        else:
            self.beam_divergence = divergence
        self.distance = distance

        if self.repetition_rate == 0:
            # Assume the laser is continuous
            self.peak_power = 0
            self.pulse_duration = 0
            self.pulse_energy = 0
            self.continuous = True
            if average_power is None:
                raise ValueError("For continous laser, average power is required.")
            self.average_power = average_power
        else:
            self.continuous = False
            (
                self.average_power,
                self.peak_power,
                self.pulse_duration,
                self.pulse_energy,
            ) = self._solve(
                repetition_rate, average_power, peak_power, pulse_duration, pulse_energy
            )

    # pylint:disable=too-many-branches
    def _solve(
        self,
        repetition_rate: float | int,
        average_power: Optional[float | int] = None,
        peak_power: Optional[float | int] = None,
        pulse_duration: Optional[float | int] = None,
        pulse_energy: Optional[float | int] = None,
    ) -> Tuple[float | int, float | int, float | int, float | int]:
        """Solve the laser parameters (from two parameters, find the other two).
        This first checks that exactly two parameters are given, and that this is
        not the (average_power, pulse_energy) case.

        Then, it manually goes over the 5 cases and use the formulas, to return all the
        parameters.

        Args:
            repetition_rate (float | int): repetition rate, in Hz.
            average_power (Optional[float  |  int], optional): average power, in W. Defaults to None.
            peak_power (Optional[float  |  int], optional): peak power, in W. Defaults to None.
            pulse_duration (Optional[float  |  int], optional): pulse duration, in s. Defaults to None.
            pulse_energy (Optional[float  |  int], optional): pulse energy, J. Defaults to None.

        Raises:
            ValueError: if stritcly more than 2 parameters are given.
            ValueError: if strictly less than 2 parameters are given.
            ValueError: if average_power and pulse_energy are the two given parameters.

        Returns:
            Tuple[float | int, float | int, float | int, float | int]: the average power in W, the peak power in W, the pulse duration in s, the pulse energy in J.
        """
        # Count number of parameters
        i = 0
        if pulse_energy is None:
            i += 1
        if average_power is None:
            i += 1
        if peak_power is None:
            i += 1
        if pulse_duration is None:
            i += 1

        if i > 2:
            raise ValueError(
                "Not enough parameters are given in (pulse energy, average power, peak power, pulse duration)"
            )
        if i < 2:
            raise ValueError(
                "Too many parameters are given in (pulse energy, average power, peak power, pulse duration)"
            )

        if average_power is not None and pulse_energy is not None:
            raise ValueError(
                "average power and pulse energy is not enough to find peak power and pulse duration"
            )

        if average_power is not None:
            if peak_power is not None:
                return (
                    average_power,
                    peak_power,
                    average_power / repetition_rate,
                    average_power / (peak_power * repetition_rate),
                )
            if pulse_duration is not None:
                return (
                    average_power,
                    average_power / (pulse_duration * repetition_rate),
                    pulse_duration,
                    average_power / repetition_rate,
                )
        if peak_power is not None:
            if pulse_duration is not None:
                return (
                    peak_power / pulse_duration * repetition_rate,
                    peak_power,
                    pulse_duration,
                    peak_power / pulse_duration,
                )
            if pulse_energy is not None:
                return (
                    repetition_rate * pulse_energy,
                    peak_power,
                    pulse_energy / peak_power,
                    pulse_energy,
                )

        # The only case left is (pulse_duration, pulse_energy)
        return (
            repetition_rate * pulse_energy,
            pulse_energy / pulse_duration,
            pulse_duration,
            pulse_energy,
        )

    @property
    def beam_diameter(self) -> float:
        """The beam diameter at the considered distance.
        If the divergence is 0, this is directly the beam
        diameter. Otherwise it is computed with

        :math:`D(d) = D_{base} + 2d\\tan(\\theta)`

        where :math:`D(d)` is the diameter at distance
        :math:`d`, :math:`D_{base}` the base diameter and
        :math:`\\theta` the divergence.

        Returns:
            float: the diameter, in m at the considered distance.
        """
        if self.beam_divergence == 0:
            return self.base_beam_diameter

        return self.base_beam_diameter + 2 * self.distance * np.tan(
            self.beam_divergence
        )

    @property
    def cross_section_area(self) -> float:
        """The cross section area, computed as

        :math:`\\pi \\left(\\frac{D(d)}{2}\\right)^2`

        where :math:`D(d)` is the beam diameter at
        distance :math:`d`.

        Returns:
            float: the cross section area, in m^2.
        """
        return np.pi * (self.beam_diameter / 2) ** 2

    @property
    def reference_number_pulses(self) -> float:
        """The reference number of pulses used in the
        EN207 standard, which is the number of pulses
        in 10 s.

        This is computed by multiplying the
        repetition rate in Hz by 10.

        Returns:
            float: number of pulses in 10 s.
        """
        return 10 * self.repetition_rate

    @property
    def power_density(self) -> float:
        """Power density in W per m^2.

        This is computed by dividing the average
        power by the cross section area.

        Returns:
            float: the average power density in W per m^2.
        """
        return self.average_power / self.cross_section_area

    @property
    def peak_power_density(self) -> float:
        """The peak power density, in W per m^2.

        This is computed by dividing the peak
        power by the cross section area.

        Returns:
            float: the peak power density in W per m^2.
        """
        return self.peak_power / self.cross_section_area

    @property
    def energy_density(self) -> float:
        """The energy density, in J per m^2.

        This is computed by dividing the pulse energy
        by the cross section area.

        Returns:
            float: the energy density, in J per m^2.
        """
        return self.pulse_energy / self.cross_section_area

    @property
    def corrected_energy_density(self) -> float:
        """The corrected energty density, in J per m^2.

        This is done by correcting the energy density
        byu multiplying it by the fourth root
        of the reference number of pulses.

        Warning: this does not perform any check on wavelength.

        Returns:
            float: the corrected energy density, in J per m^2.
        """
        return self.energy_density * self.reference_number_pulses**0.25

    @property
    def mode(self) -> Mode:
        """The mode of the laser, depending on the pulse duration.

        Returns:
            Mode: the mode of the laser.
        """
        return get_test_by_pulse_duration(self.pulse_duration)

    def __str__(self):
        return f"""
Continuous: {self.continuous}
Wavelength: {format_value(self.wavelength)}m
Repetition rate: {format_value(self.repetition_rate)}Hz
Average power: {format_value(self.average_power)}W
Peak power: {format_value(self.peak_power)}W
Pulse duration: {format_value(self.pulse_duration)}s
Pulse energy: {format_value(self.pulse_energy)}J
Beam diameter: {format_value(self.beam_diameter)}m
Mode: {str(self.mode)}
Cross section area: {self.cross_section_area} m^2
Power density: {self.power_density} W/m^2
Peak power density: {self.peak_power_density} W/m^2
Energy density: {self.energy_density} J/m^2
Corrected energy density: {self.corrected_energy_density} J/m^2
Number of pulses in 10s: {self.reference_number_pulses}
"""

    def en207_analysis(self) -> List[Tuple[Mode, int]]:
        """Perform the EN207 analysis.

        If the mode of the laser is M, it only performs the
        continuous scale analysis.

        If the mode of the laser is anything but M, it
        perform both the continuous scale and pulsed
        scale analysis.

        Returns:
            List[Tuple[Mode, int]]: list of mode and scale number required for the mode.
        """
        scale_number_cw = continuous_scale_number(
            self.average_power, self.cross_section_area, self.wavelength
        )
        if self.mode == Mode.D:
            # The D is the only analysis to do
            return [
                (Mode.D, scale_number_cw),
            ]

        # Perform analysis for pulsed
        scale_number_pulsed = pulsed_scale_number(
            self.peak_power,
            self.pulse_energy,
            self.cross_section_area,
            self.pulse_duration,
            self.repetition_rate,
            self.wavelength,
        )

        return [(Mode.D, scale_number_cw), (self.mode, scale_number_pulsed)]

    def en208_analysis(self) -> int:
        """Perform the EN208 analysis.

        Depending on the pulse duration, the continuous or pulsed computation used used.

        Returns:
            int: scale number for EN208.
        """
        # Continuous
        if self.continuous or self.pulse_duration >= 1e-4:
            return continuous_scale_number_en208(
                self.average_power, self.cross_section_area, self.wavelength
            )

        # Pulsed
        return pulsed_scale_number_en208(
            self.peak_power,
            self.pulse_energy,
            self.cross_section_area,
            self.pulse_duration,
            self.repetition_rate,
            self.wavelength,
        )



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Compute scale numbers using the EN207 standard.
"""

from typing import Union

import numpy as np

from lasersafety.modes import Mode, get_test_by_pulse_duration
from lasersafety.utils import (
    validate_wavelength_en207,
    validate_strictly_positive_number,
)


def continuous_scale_number(
    average_power: Union[float, int],
    area: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Find the scale number for a continuous beam. This
    method uses the average power (in W), the area (in m^2)
    and the wavelength in m. The power density is computed by
    dividing the average power by the area. Depending on the
    wavelength, the scale number is returned.

    Args:
        average_power (Union[float, int]): average power, in W.
        area (Union[float, int]): beam area, in m^2.
        wavelength (Union[float, int]): wavelength, in m.

    Returns:
        int: scale number for the continuous beam.
    """
    # Checks
    validate_wavelength_en207(wavelength)
    validate_strictly_positive_number(average_power, "averge power")
    validate_strictly_positive_number(area, "area")

    # Compute the density
    power_density = average_power / area

    # Use the formula depending on the wavelength

    # 180nm -315 nm
    if wavelength <= 315e-9:
        # log(P)+3
        res = np.log10(power_density) + 3

    # > 315 nm - 1400 nm
    elif wavelength <= 1400e-9:
        # log(P)-1
        res = np.log10(power_density) - 1

    # 1400 nm - 1 mm
    else:
        # log(P) - 3
        res = np.log10(power_density) - 3

    return np.max((1, np.ceil(res))).astype(int)


# pylint: disable=too-many-arguments, too-many-positional-arguments
def pulsed_scale_number(
    peak_power: Union[float, int],
    pulse_energy: Union[float, int],
    area: Union[float, int],
    pulse_duration: Union[float, int],
    repetition_rate: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Compute the scale number for a pulsed beam. This is
    done by using the peak power (in W), the pulse energy
    (in J), the area (in m^2), the pulse duration (in s),
    the repetition rate (in Hz) and the wavelength (in m).

    The laser mode is computed using the pulse duration. Then,
    it calls either :py:func:`~lasersafety.en207._pulsed_scale_number_ir`
    or :py:func:`~lasersafety.en207._pulsed_scale_number_m` with
    the pulse energy, the area, the repetition rate and the
    wavelength.

    Args:
        peak_power (Union[float, int]): peak power, in W.
        pulse_energy (Union[float, int]): pulse energy, in J.
        area (Union[float, int]): beam area, in m^2.
        pulse_duration (Union[float, int]): pulse duration, in s.
        repetition_rate (Union[float, int]): repetition rate, in Hz.
        wavelength (Union[float, int]): wavelength, in m.

    Raises:
        Exception: when using this function with a mode D laser.

    Returns:
        int: the scale number for the pulsed beam.
    """

    # Checks
    validate_strictly_positive_number(peak_power, "peak power")
    validate_strictly_positive_number(pulse_energy, "pulse energy")
    validate_strictly_positive_number(area, "area")
    validate_strictly_positive_number(pulse_duration, "pulse duration")
    validate_strictly_positive_number(repetition_rate, "repetition rate")
    validate_wavelength_en207(wavelength)

    mode = get_test_by_pulse_duration(pulse_duration)

    assert mode in (Mode.D, Mode.I, Mode.R, Mode.M)

    if mode == Mode.D:
        raise Exception("Cannot use this function with continuous laser")

    if mode in (Mode.I, Mode.R):
        return _pulsed_scale_number_ir(pulse_energy, area, repetition_rate, wavelength)

    # Mode is Mode.M
    return _pulsed_scale_number_m(
        peak_power, pulse_energy, area, repetition_rate, wavelength
    )


def _pulsed_scale_number_ir(
    pulse_energy: Union[float, int],
    area: Union[float, int],
    repetition_rate: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Compute the scale number for an I or R pulsed laser,
    done by using the pulse energy (in J), the beam area
    (in m^2), the repetition rate (in Hz) and the
    wavelength (in m).

    It computes the energy density by dividing the pulse
    energy by the area. Then if the wavelength is between
    400 nm and 1400 nm, the energy density is corrected
    with the fourth root of the number of pulses in 10 s.

    Args:
        pulse_energy (Union[float, int]): the pulse energy, in J.
        area (Union[float, int]): the beam area, in m^2.
        repetition_rate (Union[float, int]): the repetition rate, in Hz.
        wavelength (Union[float, int]): the wavelength, in m.

    Returns:
        int: the scale number for an I or R pulsed laser.
    """
    energy_density = pulse_energy / area

    # Correction if wavelength is between 400 nm and 1400 nm
    if 400e-9 <= wavelength <= 1400e-9:
        number_pulses = repetition_rate * 10
        energy_density *= number_pulses**0.25

    # Compute the scale number depending on the wavelength
    # 180nm -315 nm

    if wavelength <= 315e-9:
        # log(E/3)-1
        res = np.log10(energy_density / 3) - 1

    # > 315 nm - 1400 nm
    elif wavelength <= 1400e-9:
        # log(E/5)+3
        res = np.log10(energy_density / 5) + 3

    # 1400 nm - 1 mm
    else:
        # log(E)-2
        res = np.log10(energy_density) - 2

    return np.max((1, np.ceil(res))).astype(int)


def _pulsed_scale_number_m(
    peak_power: Union[float, int],
    pulse_energy: Union[float, int],
    area: Union[float, int],
    repetition_rate: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Compute the scale number for an M pulsed laser,
    done by using the peak power (in W), the pulse
    energy (in J), the beam area (in m^2), the repetition
    rate (in Hz) and the wavelength (in m).

    If the wavelength is between 315 nm and 1400 nm, the
    energy, computed as the ratio of the pulse energy and
    the beam area, is used to determine the scale number.
    Additionnally, if the wavelength is between 400 nm
    and 1400 nm, the energy density is corrected by
    the fourth root of the number of pulses in 10 s.

    If the wavelength is not in this range, the peak
    power density is computed by dividing the peak
    power power by the beam area. Then the scale number
    is returned.

    Args:
        peak_power (Union[float, int]): the peak power, in W.
        pulse_energy (Union[float, int]): the pulse energy, in J.
        area (Union[float, int]): the beam area, in m^2.
        repetition_rate (Union[float, int]): the repetition rate, in Hz.
        wavelength (Union[float, int]): the wavelength, in m.

    Returns:
        int: the scale number for an M pulsed laser.
    """
    if 315e-9 < wavelength <= 1400e-9:
        energy_density = pulse_energy / area
        if 400e-9 <= wavelength:
            number_pulses = repetition_rate * 10
            energy_density *= number_pulses**0.25

        # log(E/1.5)+4
        res = np.log10(energy_density / 1.5) + 4

    else:
        peak_power_density = peak_power / area

        if wavelength <= 315e-9:
            # log(P/3) - 10
            res = np.log10(peak_power_density / 3) - 10
        else:
            # log(P)-11
            res = np.log10(peak_power_density) - 11
    return np.max((1, np.ceil(res))).astype(int)



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Compute scale numbers using the EN208 standard.
"""

from typing import Union

import numpy as np

from lasersafety.utils import (
    validate_wavelength_en208,
    validate_strictly_positive_number,
)


def continuous_scale_number_en208(
    average_power: Union[float, int],
    area: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Compute the cotninuous scale number for en208.

    It computes the power density by dividing the power density
    by the area and uses it to return the sccale number.

    Args:
        average_power (Union[float, int]): the average power, in W.
        area (Union[float, int]): the beam area, in m^2.
        wavelength (Union[float, int]): the wavelength, in m.

    Returns:
        int: the scale number for the continuous laser.
    """
    # Checks
    validate_strictly_positive_number(average_power, "average power")
    validate_strictly_positive_number(area, "area")
    validate_wavelength_en208(wavelength)

    power_density = average_power / area

    # log(P)-3
    return np.ceil(np.log10(power_density) - 3).astype(int)


# pylint: disable=too-many-arguments, too-many-positional-arguments
def pulsed_scale_number_en208(
    peak_power: Union[float, int],
    pulse_energy: Union[float, int],
    area: Union[float, int],
    pulse_duration: Union[float, int],
    repetition_rate: Union[float, int],
    wavelength: Union[float, int],
) -> int:
    """Compute the scale number for the EN208 standard
    for pulsed lasers.

    It computes the corrected energy density by using the pulse
    energy, the area and the repetion rate, then returns the
    scale number.

    Args:
        peak_power (Union[float, int]): the peak power, in W.
        pulse_energy (Union[float, int]): the pulse energy, in J.
        area (Union[float, int]): the beam area, in m^2.
        pulse_duration (Union[float, int]): the pulse duration, in s.
        repetition_rate (Union[float, int]): the repetition rate, in Hz.
        wavelength (Union[float, int]): the wavelength, in m.

    Raises:
        ValueError: if pulse duration is less or equal than 1 ns.

    Returns:
        int: the scale number for the EN208 standard.
    """
    # Checks
    validate_strictly_positive_number(peak_power, "peak power")
    validate_strictly_positive_number(pulse_energy, "pulse energy")
    validate_strictly_positive_number(area, "area")
    validate_strictly_positive_number(pulse_duration, "pulse duration")
    validate_strictly_positive_number(repetition_rate, "repetition rate")
    validate_wavelength_en208(wavelength)

    if pulse_duration >= 2e-4:
        # treat as continuous as per EN208
        return continuous_scale_number_en208(
            pulse_energy * repetition_rate, area, wavelength
        )

    if pulse_duration <= 1e-9:
        raise ValueError("EN208 standard consider pulse duration > 1ns.")

    # Compute corrected energy density
    energy_density = pulse_energy / area
    number_pulses = repetition_rate * 10
    energy_density *= number_pulses**0.25

    # log(E/2)+1
    return np.ceil(np.log10(energy_density / 2) + 1).astype(int)



# Copyright (c) 2025 Yoann Piétri
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
Modes of the laser as defined in the EN207 standard.
"""

from typing import Union
from enum import StrEnum


class Mode(StrEnum):
    """
    Possible modes of the laser as defined
    in EN207 standard.
    """

    D = "D"  #: Continuous t > 0.25s
    I = "I"  #: Long pulse 1ms <= t < 0.25s
    R = "R"  #: Short pulse 1ns <= t < 1ms
    M = "M"  #: Ultra short pulse t < 1ns


def get_test_by_pulse_duration(pulse_duration: Union[float, int]) -> Mode:
    """Return the mode of the laser depending on the pulse
    duration. The pulse duration is expected in second. It
    should be a float (or an int) greater or equal than 0.
    A value of 0 is assumed to mean "continuous wave (CW)"
    and mode D is returned.

    Args:
        pulse_duration (float): duration of the pulse in seconds.

    Raises:
        ValueError: if the pulse duration is not a float (or an int).
        ValueError: if the pulse duration is strictly lower than 0.

    Returns:
        Mode: the mode corresponding to the given pulse duration as per EN207.
    """
    if not isinstance(pulse_duration, (float, int)):
        raise ValueError("pulse_duration must be a float (or int)")

    if pulse_duration < 0:
        raise ValueError(
            f"pulse_duration must be greater or equal than 0 (got {pulse_duration})"
        )

    if pulse_duration == 0:
        return Mode.D

    if pulse_duration < 1e-9:
        return Mode.M

    if pulse_duration < 1e-3:
        return Mode.R

    if pulse_duration < 0.25:
        return Mode.I

    return Mode.D
